/**
 * @file
 * Webform Autosave behaviors.
 */

(function($, Drupal, debounce) {
  'use strict';
  // Set our primary store.
  var store = {
    changedTimestamp: null,
    csrfToken: null,
    optimisticLocking: drupalSettings.webformautosave.optimistic_locking === 1,
    sid: drupalSettings.webformautosave.sid,
    submissionData: {
      webform_id: drupalSettings.webformautosave.webform_id,
    },
    submissionUrl: drupalSettings.webformautosave.submission_url,
    webformId: drupalSettings.webformautosave.webform_id,
  };

  /**
   * Getter for the CSRF Token.
   *
   * @param callback
   */
  var getCsrfToken = function(callback) {
    // Get the CSRF token if not set, then perform the callback.
    if (store.csrfToken === null) {
      $.get(Drupal.url('session/token')).done(function(data) {
        // Set the token to the returned data.
        store.csrfToken = data;
        callback(store.csrfToken);
      });
    }
    else {
      callback(store.csrfToken);
    }
  };

  /**
   * Getter for the webform submission data.
   */
  var getSubmission = function(csrfToken, updateData = {}) {
    var sid = store.sid;
    var webformId = store.webformId;
    // Perform a PATCH if there is an SID.
    var getUrl =
      '/webformautosave/' + webformId + '/submission/' + sid + '?_format=json';
    $.ajax({
      url: getUrl,
      method: 'GET',
      headers: {
        'Content-Type': 'application/json',
        'X-CSRF-Token': csrfToken,
      },
    }).done(function(response) {
      // Get the changed time.
      var changedTime = Date.parse(response.entity.changed[0].value);
      if (Object.keys(updateData).length !== 0) {
        if (store.optimisticLocking && store.changedTimestamp && changedTime > store.changedTimestamp) {
          setMessage('The content has either been modified by another user, or you have already submitted modifications. As a result, your changes cannot be saved. <a href="' + store.submissionUrl + '">Refresh this form</a> to continue editing.')
          console.error('The content has been modified by another user.');
        }
        else {
          store.changedTimestamp = changedTime;
          updateSubmission(csrfToken, updateData);
        }
      }
      else {
        store.changedTimestamp = changedTime;
        store.submissionData = response.data;
      }
    });
  };
  /**
   * Set a Drupal message.
   */
  var setMessage = function(message, status = 'error') {
    var error = '<div role="contentinfo" aria-label="Error message" class="messages message-body messages--' + status + '">' + message + '</div>';
    if ($('.messages__wrapper').length) {
      $('.messages__wrapper').prepend(error);
    }
    else if ($('main').length) {
      $('main').prepend('<div class="message is-danger">' + error + '</div>');
    }
    else {
      $('.main-content').prepend('<div class="message is-danger">' + error + '</div>')
    }
    window.scrollTo(0, 0);
  };

  /**
   * Updater for the webform submission.
   */
  var updateSubmission = function(csrfToken, data) {
    var sid = store.sid;
    var webformId = store.webformId;
    // Perform a PATCH if there is an SID.
    var patchUrl =
      '/webformautosave/' + webformId + '/submission/' + sid + '?_format=json';
    $.ajax({
      url: patchUrl,
      method: 'PATCH',
      headers: {
        'Content-Type': 'application/json',
        'X-CSRF-Token': csrfToken,
      },
      data: JSON.stringify(data),
      error: function (request, status, error) {
        setMessage(request.responseText);
      }
    }).done(function(response) {
      // Get the current submission data.
      getCsrfToken(function(csrfToken) {
        getSubmission(csrfToken);
      });
    });
  };

  /**
   * Posts a webform submission.
   */
  var postSubmission = function(csrfToken) {
    // Set our webform ID.
    store.submissionData['webform_id'] = store.webformId;
    // Perform a PATCH if there is an SID.
    var postUrl = '/webformautosave/submit?_format=json';
    $.ajax({
      url: postUrl,
      method: 'POST',
      headers: {
        'Content-Type': 'application/json',
        'X-CSRF-Token': csrfToken,
      },
      data: JSON.stringify(store.submissionData),
    }).done(function(response) {
      store.submissionData = response.data;
      store.sid = response.sid;
    });
  };

  /**
   * Debounced action to perform on change.
   */
  var changeAction = debounce(
    function(event) {
      var value = event.target.value;
      var name = event.target.name;
      var newData = {};
      newData[name] = value;
      // Patch the submission if the form has an SID.
      getCsrfToken(function(csrfToken) {
        getSubmission(csrfToken, newData);
      });
    },
    250,
    false
  );

  /**
   * Setup our default behaviors for the webformautosave module.
   */
  Drupal.behaviors.webformautosave = {
    // eslint-disable-next-line no-unused-vars
    attach: function(context, settings) {
      // Let's bind an input event to our inputs once.
      $('form.webform-submission-form', context)
        .once('webformAutosaveBehavior')
        .each(function() {
          // Update our stored submission data.
          if (store.sid) {
            // Get the current submission data.
            getCsrfToken(function(csrfToken) {
              getSubmission(csrfToken);
            });
          }
          // Add an input event listener to each input.
          $(this)
            .find('input, select')
            .each(function() {
              $(this).change(changeAction);
            });
        });
    },
  };
})(jQuery, Drupal, Drupal.debounce);
